function initial_map = construct_rect_initial_map(v, f, corner, flatten_method, aspect_ratio)
% Compute rectangular initial map using Tutte/locally authalic method
%
% If you use this code in your own work, please cite the following paper:
% [1] G. P. T. Choi and C. H. Rycroft, 
%     "Density-equalizing maps for simply connected open surfaces."
%     SIAM Journal on Imaging Sciences, 11(2), pp. 1134-1178, 2018.
% 
% Copyright (c) 2017-2018, Gary Pui-Tung Choi
% https://scholar.harvard.edu/choi

% for surface flattening, 1 = Tutte, 2 = locally authalic
if ~exist('flatten_method','var')
    flatten_method = 1;
end

% for transforming the initial map to achieve the desired shape
if ~exist('aspect_ratio','var')
    aspect_ratio = [1, 1];
end

if flatten_method == 1
    % tutte map
    initial_map = rect_tutte_map(v,f,corner,aspect_ratio);
else
    % locally authalic map
    initial_map = rect_locally_authalic_map(v,f,corner,aspect_ratio);

    % check bijectivity
    a = initial_map(f(:,2),:) - initial_map(f(:,1),:);
    b = initial_map(f(:,3),:) - initial_map(f(:,1),:);
    normal_direction = ((a(:,1).*b(:,2) - a(:,2).*b(:,1)) > 0);
    if length(unique(normal_direction)) > 1
       % contain overlaps, switch to tutte map
       initial_map = rect_tutte_map(v,f,corner,aspect_ratio);
    end
end