function kappa = compute_curve_curvature(X)
% Compute the curvature of the curve described by the polyline in X in R^3.
% INPUT: X, a polyline X = [x1,y1,z2; x2,y2,z2; ....; xN,yN,zN]
% OUTPUT:
% kappa: the curvature
% Reference:
% https://www.mathworks.com/matlabcentral/newsreader/view_thread/77421

T = diff(X); % dX/dt
ds = sqrt(sum(T.*T,2)); 

T = T./ds(:,[1,1,1]); % T = dX/ds = dX/dt.*dt/ds
N = diff(T); % N // dT
ds(end) = [];
kappa = sqrt(sum(N.*N,2)); % |N|
kappa = kappa./ds; % T' = kappa*ds*N => kappa = |T'|/ds
kappa(end) = [];