function map = locally_authalic_map(v, f, bdyid, bdy)
% Compute the locally authalic map.
% 
% Input:
% vertex: nx3 vertex coordinates
% face: mx3 triangulation
% bdy_index : kx1 array of bdy indices
% bdy: target bdy in complex coordinates
% 
% Output:
% map: nx2 coordinates
%
% If you use this code in your own work, please cite the following paper:
% [1] G. P. T. Choi and C. H. Rycroft, 
%     "Density-equalizing maps for simply connected open surfaces."
%     SIAM Journal on Imaging Sciences, 11(2), pp. 1134-1178, 2018.
% 
% Copyright (c) 2017-2018, Gary Pui-Tung Choi
% https://scholar.harvard.edu/choi

nv = length(v);

M = authalic_matrix(v,f); 

[mrow,mcol,mval] = find(M(bdyid,:));
M = M - sparse(bdyid(mrow),mcol,mval,nv, nv) + ...
        sparse(bdyid,bdyid,ones(length(bdyid),1),nv, nv);
    
% boundary constraint
c = zeros(nv,1); 
c(bdyid) = bdy;

% solve equation
z = M \ c;
map = [real(z), imag(z)];