function uv = rect_locally_authalic_map(v,f,corner,aspect_ratio)
% Compute the rectangular locally authalic map.
%
% Input:
% v: nv x 3 
% f: nf x 3
% corner: 4 x 1 corner indices 
% 4 - 3
% |   |
% 1 - 2
% aspect_ratio: target aspect ratio
% 
% Output:
% uv: nv x 2 uv coordinates
%
% If you use this code in your own work, please cite the following paper:
% [1] G. P. T. Choi and C. H. Rycroft, 
%     "Density-equalizing maps for simply connected open surfaces."
%     SIAM Journal on Imaging Sciences, 11(2), pp. 1134-1178, 2018.
% 
% Copyright (c) 2017-2018, Gary Pui-Tung Choi
% https://scholar.harvard.edu/choi

nv = length(v);

TR = TriRep(f,v); B = freeBoundary(TR); bd = B(:,1);
nbd = size(bd,1);


i = find(bd==corner(1),1,'first');
bd = bd([i:end,1:i]);
corner = corner([1:end,1]);

ck = zeros(size(corner));
k = 1;
for i = 1:length(bd)
    if(bd(i) == corner(k))
        ck(k) = i;
        k = k+1;
    end
end
uvbd = zeros(nbd,2);
uvbd(ck(1):ck(2),1) = linspace(0,1,ck(2)-ck(1)+1)'*aspect_ratio(1);
uvbd(ck(1):ck(2),2) = 0;
uvbd(ck(2):ck(3),1) = 1*aspect_ratio(1);
uvbd(ck(2):ck(3),2) = linspace(0,1,ck(3)-ck(2)+1)'*aspect_ratio(2);
uvbd(ck(3):ck(4),1) = linspace(1,0,ck(4)-ck(3)+1)'*aspect_ratio(1);
uvbd(ck(3):ck(4),2) = 1*aspect_ratio(2);
uvbd(ck(4):ck(4),1) = 0;
uvbd(ck(4):ck(5),2) = linspace(1,0,ck(5)-ck(4)+1)'*aspect_ratio(2);

uvbd(end,:) = [];    
bd(end) = [];
uv = zeros(nv,2);
uv(bd,:) = uvbd;
in = true(nv,1);
in(bd) = false;

% construct matrix
A = authalic_matrix(v,f); 
fixed = bd;
[mrow,mcol,mval] = find(A(fixed,:));
A = A - sparse(fixed(mrow),mcol,mval,nv, nv) + ...
        sparse(fixed,fixed,ones(length(fixed),1),nv, nv);
Ain = A(in,in);
b = -A(in,bd)*uvbd;

% solve equation
uvin = Ain\b;
uv(in,:) = uvin;
