function map = tutte_map(v, f, bdyid, bdy)
% Compute the Tutte map.
%
% Input:
% v: nv x 3 
% f: nf x 3
% corner: 4 x 1 corner indices 
% 4 - 3
% |   |
% 1 - 2
% aspect_ratio: target aspect ratio
% 
% Output:
% uv: nv x 2 uv coordinates
%
% If you use this code in your own work, please cite the following paper:
% [1] G. P. T. Choi and C. H. Rycroft, 
%     "Density-equalizing maps for simply connected open surfaces."
%     SIAM Journal on Imaging Sciences, 11(2), pp. 1134-1178, 2018.
% 
% Copyright (c) 2017-2018, Gary Pui-Tung Choi
% https://scholar.harvard.edu/choi

nv = length(v);
nf = length(f);

% compute adjacency matrix for tutte embedding
row = [f(:,1);f(:,2);f(:,3)]';
col = [f(:,2);f(:,3);f(:,1)]';
val = ones(1,3*nf)/2;
W = sparse(row,col,val,nv,nv);
M = W + sparse(1:nv,1:nv,-diag(W)-(sum(W)'), nv, nv);

[mrow,mcol,mval] = find(M(bdyid,:));
M = M - sparse(bdyid(mrow), mcol, mval, nv, nv) + ...
        sparse(bdyid, bdyid, ones(length(bdyid),1), nv, nv);
    
% boundary constraint
b = zeros(nv,1);
b(bdyid) = bdy;

% solve equation
z = M\b;
map = [real(z),imag(z)];