% DEM_main: Compute the density-equalizing map of a simply-connected open surface based on a prescribed population (with free boundary condition)
% DEM_rect: Compute the density-equalizing map of a simply-connected open surface based on a prescribed population (with square/rectangular boundary condition)
% DEM_disk: Compute the density-equalizing map of a simply-connected open surface based on a prescribed population (with disk/elliptic boundary condition)
% 
% If you use this code in your own work, please cite the following paper:
% [1] G. P. T. Choi and C. H. Rycroft, 
%     "Density-equalizing maps for simply connected open surfaces."
%     SIAM Journal on Imaging Sciences, 11(2), pp. 1134-1178, 2018.
% 
% Copyright (c) 2017-2018, Gary Pui-Tung Choi
% https://scholar.harvard.edu/choi

addpath(genpath('code')) 
addpath('data')

%% %%%%%%%%%%%%%%%%%%%%%% %%
%            DEM           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Square 
load('square.mat')

figure; 
patch('Faces',f,'Vertices',v,'FaceColor','flat','FaceVertexCData',population);
colormap jet; colorbar
axis equal tight
title('Input')

% initial map
initial_map = construct_initial_map(v,f);

% density-equalizing map
map = DEM_main(v,f,population,initial_map);

figure; 
patch('Faces',f,'Vertices',map,'FaceColor','w'); 
axis equal tight
title('Result')

%% Hexagon 
load('hexagon.mat'); 

figure; 
patch('Faces',f,'Vertices',v,'FaceColor','flat','FaceVertexCData',population);
colormap jet; colorbar
axis equal tight
title('Input')

% initial map
initial_map = construct_initial_map(v,f);

% density-equalizing map
map = DEM_main(v,f,population,initial_map);

figure; 
patch('Faces',f,'Vertices',map,'FaceColor','w'); 
axis equal tight
title('Result')

%% peaks
load('peaks.mat')

figure; 
patch('Faces',f,'Vertices',v,'FaceColor','flat','FaceVertexCData',population);
colormap jet; colorbar
axis equal tight
title('Input')
view([20 20])

% initial map
initial_map = construct_initial_map(v,f);

% density-equalizing map
map = DEM_main(v,f,population,initial_map);

figure; 
patch('Faces',f,'Vertices',map,'FaceColor','w'); 
axis equal tight
title('Result')

%% %%%%%%%%%%%%%%%%%%%%%% %%
%   Shape-prescribed DEM   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% square to square
load('square.mat')
corner = [10 1 91 100];

figure; 
patch('Faces',f,'Vertices',v,'FaceColor','flat','FaceVertexCData',population);
colormap jet; colorbar
axis equal tight
title('Input')

% initial map
initial_map = construct_rect_initial_map(v, f, corner, 1, [1 1]);

% shape-prescribed density-equalizing map
map = DEM_rect(v,f,population,initial_map,corner);

figure; 
patch('Faces',f,'Vertices',map,'FaceColor','w'); 
axis equal tight
title('Result')

%% square to rectangle
load('square.mat')
corner = [10 1 91 100];

figure; 
patch('Faces',f,'Vertices',v,'FaceColor','flat','FaceVertexCData',population);
colormap jet; colorbar
axis equal tight
title('Input')

% initial map
initial_map = construct_rect_initial_map(v, f, corner, 1, [2 1]);

% shape-prescribed density-equalizing map
map = DEM_rect(v,f,population,initial_map,corner);

figure; 
patch('Faces',f,'Vertices',map,'FaceColor','w'); 
axis equal tight
title('Result')

%% peaks to disk
load('peaks.mat')

figure; 
patch('Faces',f,'Vertices',v,'FaceColor','flat','FaceVertexCData',population);
colormap jet; colorbar
axis equal tight
title('Input')
view([20 20])

% initial map
initial_map = construct_disk_initial_map(v,f);

% shape-prescribed density-equalizing map
map = DEM_disk(v,f,population,initial_map);

figure; 
patch('Faces',f,'Vertices',map,'FaceColor','w'); 
axis equal
title('Result')
